@@ -42,8 +42,8 @@ from nuitka.utils.FileOperations import (
     areInSamePaths,
     getNormalizedPath,
     isFilenameBelowPath,
-    listDir,
     makePath,
+    relpath,
 )
 from nuitka.utils.SharedLibraries import copyDllFile, setSharedLibraryRPATH
 from nuitka.utils.Signing import addMacOSCodeSignature
@@ -177,9 +177,7 @@ def _detectBinaryDLLs(
         )
 
 
-def copyDllsUsed(dist_dir, standalone_entry_points, data_file_paths):
-    # This is complex, because we also need to handle OS specifics.
-
+def copyDllsUsed(dist_dir, standalone_entry_points):
     # Only do ones not ignored
     copy_standalone_entry_points = [
         standalone_entry_point
@@ -232,36 +230,86 @@ def copyDllsUsed(dist_dir, standalone_entry_points, data_file_paths):
 
     closeProgressBar()
 
-    # Make all top level directories symlinks for signing issues with MacOS
-    # bundles.
-    if shallCreateAppBundle():
-        resources_dir = getNormalizedPath(os.path.join(dist_dir, "..", "Resources"))
-        makePath(resources_dir)
-
-        for fullpath, filename in listDir(dist_dir):
-            if os.path.isdir(fullpath) and not os.path.islink(fullpath):
-                makePath(resources_dir)
-
-                resources_path = os.path.join(resources_dir, filename)
-
-                os.rename(fullpath, resources_path)
-                os.symlink(os.path.join("..", "Resources", filename), fullpath)
-
     Plugins.onCopiedDLLs(
         dist_dir=dist_dir,
         standalone_entry_points=copy_standalone_entry_points,
     )
 
-    # Add macOS code signature
-    if isMacOS():
-        addMacOSCodeSignature(
-            filenames=[
+    return main_standalone_entry_point, copy_standalone_entry_points
+
+
+def signDistributionMacOS(
+    dist_dir, data_file_paths, main_standalone_entry_point, copy_standalone_entry_points
+):
+    # Make all top level directories symlinks for signing issues with MacOS
+    # bundles. This is complex, because we also need to handle the need to
+    # symlink and track information. pylint: disable=too-many-locals
+
+    translations = OrderedSet()
+    symlinks = OrderedSet()
+
+    def _translatePath(path):
+        for translation in translations:
+            path = path.replace(translation[0] + "/", translation[1] + "/", 1)
+
+        return path
+
+    if shallCreateAppBundle():
+        app_path = getNormalizedPath(os.path.join(dist_dir, "..", ".."))
+        resources_dir = getNormalizedPath(os.path.join(dist_dir, "..", "Resources"))
+
+        def getNotSignableDirectoryPart(filename):
+            result = []
+
+            for part in os.path.dirname(filename).split("/"):
+                result.append(part)
+                if "." in part:
+                    return "/".join(result)
+
+            return None
+
+        for data_file_path in sorted(data_file_paths, key=len):
+            data_file_path = _translatePath(data_file_path)
+
+            inside_path = relpath(data_file_path, start=app_path)
+
+            if not inside_path.startswith("Contents/MacOS"):
+                continue
+
+            not_signable_part = getNotSignableDirectoryPart(inside_path)
+            if not_signable_part is None:
+                continue
+
+            filename = not_signable_part[len("Contents/MacOS/") :]
+            not_signable_path = os.path.join(app_path, not_signable_part)
+            resources_path = os.path.join(resources_dir, filename)
+
+            if resources_path in symlinks:
+                continue
+
+            makePath(os.path.dirname(resources_path))
+            os.rename(not_signable_path, resources_path)
+
+            symlink_target = os.path.join("..", "Resources", filename)
+            for _i in range(filename.count("/")):
+                symlink_target = os.path.join("..", symlink_target)
+
+            os.symlink(symlink_target, not_signable_path)
+
+            symlinks.add(resources_path)
+            translations.add((not_signable_path, resources_path))
+
+    addMacOSCodeSignature(
+        filenames=[
+            _translatePath(filename)
+            for filename in [
                 os.path.join(dist_dir, standalone_entry_point.dest_path)
                 for standalone_entry_point in [main_standalone_entry_point]
                 + copy_standalone_entry_points
             ]
             + data_file_paths
-        )
+        ]
+    )
 
 
 _excluded_system_dlls = set()
